%% Matlab file to calculate and plot the
% -------------------------------------------------------------------
% Matsuoka-Nakai failure surface
% written by Gertraud Medicus within the project 
% 'Animating Soil Models' - 01/2021
% University of Innsbruck
% -------------------------------------------------------------------
% useful information is in e.g. Griffiths & Huang (2009):
% 'Observations on the extended Matsuoka–Nakai failure criterion'
% https://doi.org/10.1002/nag.810
% -------------------------------------------------------------------
close all
clear all
clc
disp('================================================')
disp('                                                ')
disp('     Matsuoka-Nakai (MN) failure surface        ')
disp('           for phi=phi_c and c=0                ')
disp('          Gertraud Medicus (01/2021)            ')
disp('     https://soilmodels.com/soilanim/           ')
disp('                                                ')
disp('================================================')
%% choose friction angle
phi = 30*pi/180; % in radian
K_MN= (9-sin(phi)^2)/(1-sin(phi)^2); % MN constant

%% apply Newton Iteration to determine invariant (II) 't' to obtain yield surface
% f=0 according to Matsuoka-Nakai
% dermine Lode coordinates (e.g. Griffiths & Huang (2009) eqs. 5-8)
Theta = 1:361;  % Lode angle
j=1;
for p=[1 100*sqrt(3)] % mean effective stress p, 2 values
    for i = Theta
 %% Lode invariants       
        s=3*p/sqrt(3); % invariant (I) s, eq. 6
        t0 =3*p/4; t(j) = t0; % choose initial value for invariant (II) t
        theta = i*pi/180; % invariant (III) Lode angle theta
       
        % yield function MN f, initial value: 
        f=sqrt(2)*K_MN*sin(3*theta)*t(j)^3+(9-3*K_MN)*t(j)^2*s+(2*K_MN-18)*s^3;
 
        dt=1.0e-7; % small value delta_t for the numerical differentiation
        nr=1;

        %% Newton Iteration to determine t (eq. 7) in order to obtain f=0 (eq. 10)
        while (abs(f) > 1.0e-6) && (nr<=10)
            % Matsuoka-Nakai:
            % get f(t) Griffiths & Huang (2009) eq. (10)
            f=-sqrt(2)*K_MN*sin(3*theta)*t(j)^3+(9-3*K_MN)*t(j)^2*s+(2*K_MN-18)*s^3;
            % get f(t + dt): 
            t_dt=t(j)+dt; % t + dt
            f_dt=-sqrt(2)*K_MN*sin(3*theta)*t_dt^3+(9-3*K_MN)*t_dt^2*s+(2*K_MN-18)*s^3;
            
            % Numerical differentiation
            diff_f_t = -1/dt * (f_dt - f);	%
            
            % new value t, continue Newton iteration until 'f=0'
            t(j) = t(j) + f/diff_f_t;
            
            nr=nr+1;
        end
        %% e.g. Griffiths & Huang (2009) eq. (11)
        sigma(j,1) = s/sqrt(3)+sqrt(2/3)*t(j)*sin(theta-2*pi/3);
        sigma(j,2) = s/sqrt(3)+sqrt(2/3)*t(j)*sin(theta);
        sigma(j,3) = s/sqrt(3)+sqrt(2/3)*t(j)*sin(theta+2*pi/3);
        
        j=j+1;
    end
    
end
%% reshape coordinates for 3D plot
N=2;siz = size(sigma(:,1)',2)/N;
x = reshape(sigma(:,2)',siz,N);
y = reshape(sigma(:,1)',siz,N);
z = reshape(sigma(:,3)',siz,N);
%% 3D plot
figure(1)
set(gcf,'defaultaxesfontsize',10);
set(gcf,'defaulttextfontsize',10);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [8.3 8.3]);
set(gcf, 'PaperPosition', [0 0 8.3 8.3]);
% choose colors [red green blue]
cone_clr=[.8 .2 .1]; % color cone
line_clr=[.9 .5 0]; % color line
iso_clr=[0 0.2 .4]; % color iso ax
%
%% 3D surface, facealpha to control transparency 
colormap(cone_clr) % colormap cone
surf(x,y,z,'FaceAlpha',1); hold on
plot3(x(end-360:end),y(end-360:end),z(end-360:end),'-','Color',line_clr); hold on
%plot3(x,y,z,'-','Color',line_clr); hold on
%% hydrostatic axis
iso=[0,0,0;350,350,350]';
plot3(iso(1,:),iso(2,:),iso(3,:),'-','Color',iso_clr,'Linewidth',.5)
%% axes 
plot3([-10 350],[0 0],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[-10 350],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[0 0],[-10 350],'-','Color','k','Linewidth',.5);
text(0,0,360,'$$\sigma_1''$$', 'Interpreter','Latex');hold on
text(360,0,0,'$$\sigma_2''$$', 'Interpreter','Latex');hold on
text(0,360,0,'$$\sigma_3''$$', 'Interpreter','Latex');hold on

%% or use default Matlab axes
% Ticks = 0:500:400;
% set(gca, 'XTickMode', 'manual', 'XTick', Ticks, 'xlim', [0 360]);
% set(gca, 'YTickMode', 'manual', 'YTick', Ticks, 'ylim', [0 360]);
% set(gca, 'ZTickMode', 'manual', 'ZTick', Ticks, 'zlim', [0 360]);
% xlabel('$$\sigma_2''$$', 'Interpreter','Latex');
% ylabel('$$\sigma_3''$$', 'Interpreter','Latex');
% zlabel('$$\sigma_1''$$', 'Interpreter','Latex');

axis equal; grid off; axis off; 
set(gca,'LooseInset',get(gca,'TightInset')); % remove border around figure
%
xlim([-10 360]);
ylim([-10 360]);
zlim([-10 360]);

view([80 15]);
% view([135 35]); % view deviatoric plane

%% light settings
camlight('left')
shading interp
camlight(-30,-atand(1/sqrt(2)))
set(gcf,'Renderer','zbuffer')
set(findobj(gca,'type','surface'),...
    'FaceLighting','phong',...
    'AmbientStrength',.3,'DiffuseStrength',.8,...
    'SpecularStrength',.9,'SpecularExponent',25,...
    'BackFaceLighting','unlit')

%% print figure 
print(gcf,'-r300','-dpng','MN.png')